use crate::util::i32_from_string;
use serde::Deserialize;
use crate::{ApiError, LastfmApiError, LastfmClient};
use crate::user::Track;

#[derive(Deserialize, Clone, Debug)]
pub struct GetRecentTracksResponse {
    pub recenttracks: RecentTracks,
}

#[derive(Deserialize, Clone, Debug)]
pub struct RecentTracks {
    pub track: Vec<Track>,
    #[serde(rename(deserialize = "@attr"))]
    pub attributes: GetRecentTracksAttributes,
}

#[derive(Deserialize, Clone, Debug)]
#[serde(rename_all = "camelCase")]
pub struct GetRecentTracksAttributes {
    pub user: String,
    #[serde(deserialize_with = "i32_from_string")]
    pub total_pages: i32,
    #[serde(deserialize_with = "i32_from_string")]
    pub page: i32,
    #[serde(deserialize_with = "i32_from_string")]
    pub per_page: i32,
    #[serde(deserialize_with = "i32_from_string")]
    pub total: i32
}

pub struct GetRecentTracks;

impl GetRecentTracks {
    pub async fn send(lastfm_client: &LastfmClient, username: String, limit: Option<i32>, page: Option<i32>, from: Option<i32>, to: Option<i32>, extended: bool) -> Result<GetRecentTracksResponse, LastfmApiError> {
        let mut params: Vec<(&str, String)> = Vec::new();

        params.push(("user", username));

        if let Some(l) = limit {
            params.push(("limit", l.to_string()));
        }

        if let Some(p) = page {
            params.push(("page", p.to_string()));
        }

        if let Some(f) = from {
            params.push(("from", f.to_string()));
        }

        if let Some(t) = to {
            params.push(("to", t.to_string()));
        }

        if extended {
            params.push(("extended", "1".to_string()));
        }

        let api_url = lastfm_client.build_get_url("user.getRecentTracks", params.iter().map(| (k, v) | (*k, v.as_str())).collect::<Vec<(&str, &str)>>().as_slice(), false);

        let resp = reqwest::get(api_url.as_str())
            .await?
            .text()
            .await?;

        match serde_json::from_str::<GetRecentTracksResponse>(resp.as_str()) {
            Ok(resp) => Ok(resp),
            Err(e) => {
                println!("{:#?}", e);

                Err(serde_json::from_str::<ApiError>(resp.as_str())?.into_error())
            }
        }
    }
}