use crate::util::bool_from_string_u8;
use crate::util::i64_from_string;
use crate::util::bool_from_string;
use serde::Deserialize;

pub mod get_recent_tracks;

#[derive(Deserialize, Clone, Debug)]
pub struct Track {
    pub mbid: String,
    pub name: String,
    pub url: String,
    #[serde(deserialize_with = "bool_from_string_u8")]
    pub streamable: bool,
    pub image: Vec<Image>,
    pub artist: Option<Artist>,
    pub album: Option<MbidText>,
    pub date: Option<UnixTimestampText>,
    #[serde(default)]
    #[serde(deserialize_with = "bool_from_string_u8")]
    pub loved: bool,
    #[serde(rename(deserialize = "@attr"))]
    pub attributes: Option<TrackAttributes>,
}

#[derive(Deserialize, Clone, Debug)]
pub struct Artist {
    pub url: Option<String>,
    pub name: Option<String>,
    pub image: Option<Vec<Image>>,
    pub mbid: String,
    #[serde(rename(deserialize = "#text"))]
    pub text: Option<String>,
}

#[derive(Deserialize, Clone, Debug)]
pub struct MbidText {
    pub mbid: String,
    #[serde(rename(deserialize = "#text"))]
    pub text: String,
}

#[derive(Deserialize, Clone, Debug)]
pub struct UnixTimestampText {
    #[serde(deserialize_with = "i64_from_string")]
    pub uts: i64,
    #[serde(rename(deserialize = "#text"))]
    pub text: String,
}

#[derive(Deserialize, Clone, Debug)]
pub struct Image {
    pub size: String,
    #[serde(rename(deserialize = "#text"))]
    pub text: String,
}

#[derive(Deserialize, Clone, Debug)]
pub struct TrackAttributes {
    #[serde(deserialize_with = "bool_from_string")]
    pub nowplaying: bool,
}

impl Track {
    pub fn is_playing(&self) -> bool {
        if let Some(attr) = self.clone().attributes {
            attr.nowplaying
        } else {
            false
        }
    }
}

impl Artist {
    pub fn get_artist_name(&self) -> String {
        if self.name.is_some() {
            self.name.clone().unwrap()
        } else {
            self.text.clone().unwrap()
        }
    }
}